package com.guwan.backend.service;

import ai.djl.ndarray.NDArray;
import ai.djl.ndarray.NDList;
import ai.djl.ndarray.NDManager;
import ai.djl.training.util.ProgressBar;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

@Slf4j
@Service
public class DeepRecommendService {

    private static final int EMBEDDING_SIZE = 128;
    private static final String MODEL_PATH = "models/recommend_model.pt";

    private final NDManager manager;
    private final Model model;

    public DeepRecommendService() {
        // 初始化DJL环境
        this.manager = NDManager.newBaseManager();
        this.model = Model.newInstance("recommend");
        loadModel();
    }

    /**
     * 获取视频的嵌入向量
     */
    public float[] getVideoEmbedding(Video video) {
        try {
            // 准备输入特征
            NDArray features = prepareVideoFeatures(video);
            
            // 获取模型输出
            NDArray embedding = model.forward(new NDList(features)).singletonOrThrow();
            
            // 转换为Java数组
            return embedding.toFloatArray();
        } catch (Exception e) {
            log.error("获取视频嵌入向量失败", e);
            return new float[EMBEDDING_SIZE];
        }
    }

    /**
     * 获取用户的嵌入向量
     */
    public float[] getUserEmbedding(List<UserBehavior> behaviors) {
        try {
            // 准备输入特征
            NDArray features = prepareUserFeatures(behaviors);
            
            // 获取模型输出
            NDArray embedding = model.forward(new NDList(features)).singletonOrThrow();
            
            // 转换为Java数组
            return embedding.toFloatArray();
        } catch (Exception e) {
            log.error("获取用户嵌入向量失败", e);
            return new float[EMBEDDING_SIZE];
        }
    }

    /**
     * 计算相似度分数
     */
    public double calculateSimilarityScore(float[] embedding1, float[] embedding2) {
        double dotProduct = 0.0;
        double norm1 = 0.0;
        double norm2 = 0.0;
        
        for (int i = 0; i < embedding1.length; i++) {
            dotProduct += embedding1[i] * embedding2[i];
            norm1 += embedding1[i] * embedding1[i];
            norm2 += embedding2[i] * embedding2[i];
        }
        
        return dotProduct / (Math.sqrt(norm1) * Math.sqrt(norm2));
    }

    private void loadModel() {
        try {
            // 加载预训练模型
            Path modelPath = Paths.get(MODEL_PATH);
            model.load(modelPath);
        } catch (Exception e) {
            log.error("加载推荐模型失败", e);
        }
    }

    private NDArray prepareVideoFeatures(Video video) {
        // 将视频特征转换为模型输入格式
        float[] features = new float[]{
            video.getDuration(),
            video.getViewCount(),
            video.getLikeCount(),
            // ... 其他特征
        };
        return manager.create(features);
    }

    private NDArray prepareUserFeatures(List<UserBehavior> behaviors) {
        // 将用户行为特征转换为模型输入格式
        float[] features = new float[]{
            // 统计特征
            behaviors.size(),
            calculateAverageWatchDuration(behaviors),
            calculateLikeRatio(behaviors),
            // ... 其他特征
        };
        return manager.create(features);
    }
} 