package com.guwan.backend.service.impl;

import com.guwan.backend.service.VideoRecommendService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import java.util.*;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class VideoRecommendServiceImpl implements VideoRecommendService {

    private final UserBehaviorMapper behaviorMapper;
    private final VideoTagWeightMapper tagWeightMapper;
    private final UserInterestMapper interestMapper;
    private final VideoMapper videoMapper;
    private final DeepRecommendService deepRecommendService;

    @Override
    public List<VideoDTO> getRecommendVideos(Long userId, int limit) {
        // 1. 获取用户行为数据
        List<UserBehavior> behaviors = getRecentBehaviors(userId);
        
        // 2. 获取用户嵌入向量
        float[] userEmbedding = deepRecommendService.getUserEmbedding(behaviors);
        
        // 3. 获取候选视频
        List<Video> candidates = getCandidateVideos(userId);
        
        // 4. 计算每个视频的推荐分数
        Map<Long, Double> scores = new HashMap<>();
        for (Video video : candidates) {
            // 获取视频嵌入向量
            float[] videoEmbedding = deepRecommendService.getVideoEmbedding(video);
            
            // 计算相似度分数
            double similarityScore = deepRecommendService.calculateSimilarityScore(
                userEmbedding, videoEmbedding);
                
            // 结合其他特征计算最终分数
            double finalScore = calculateFinalScore(similarityScore, video);
            scores.put(video.getId(), finalScore);
        }
        
        // 5. 排序并返回推荐结果
        return candidates.stream()
            .sorted((v1, v2) -> Double.compare(scores.get(v2.getId()), scores.get(v1.getId())))
            .limit(limit)
            .map(this::convertToDTO)
            .collect(Collectors.toList());
    }

    @Override
    @Transactional
    public void recordUserBehavior(Long userId, Long videoId, String behaviorType, 
                                 Integer watchDuration, Float watchProgress) {
        UserBehavior behavior = new UserBehavior();
        behavior.setUserId(userId);
        behavior.setVideoId(videoId);
        behavior.setBehaviorType(behaviorType);
        behavior.setWatchDuration(watchDuration);
        behavior.setWatchProgress(watchProgress);
        
        behaviorMapper.insert(behavior);
        
        // 异步更新用户兴趣
        updateUserInterests(userId);
    }

    @Override
    @Transactional
    public void updateUserInterests(Long userId) {
        // 1. 获取用户近期行为
        List<UserBehavior> behaviors = getRecentBehaviors(userId);
        
        // 2. 计算每个标签的权重
        Map<String, Double> tagWeights = calculateTagWeights(behaviors);
        
        // 3. 更新用户兴趣表
        updateUserInterestTable(userId, tagWeights);
    }

    @Override
    public double calculateVideoSimilarity(Long videoId1, Long videoId2) {
        // 基于标签的余弦相似度计算
        List<VideoTagWeight> tags1 = tagWeightMapper.selectByVideoId(videoId1);
        List<VideoTagWeight> tags2 = tagWeightMapper.selectByVideoId(videoId2);
        
        return calculateCosineSimilarity(tags1, tags2);
    }

    @Override
    public List<VideoDTO> getSimilarVideos(Long videoId, int limit) {
        // 1. 获取视频标签
        List<VideoTagWeight> sourceTags = tagWeightMapper.selectByVideoId(videoId);
        
        // 2. 获取候选视频
        List<Video> candidates = getCandidateVideos(null);
        
        // 3. 计算相似度并排序
        Map<Long, Double> similarities = new HashMap<>();
        for (Video candidate : candidates) {
            if (!candidate.getId().equals(videoId)) {
                double similarity = calculateVideoSimilarity(videoId, candidate.getId());
                similarities.put(candidate.getId(), similarity);
            }
        }
        
        return candidates.stream()
            .filter(v -> !v.getId().equals(videoId))
            .sorted((v1, v2) -> Double.compare(similarities.get(v2.getId()), similarities.get(v1.getId())))
            .limit(limit)
            .map(this::convertToDTO)
            .collect(Collectors.toList());
    }

    private double calculateFinalScore(double similarityScore, Video video) {
        // 结合多个因素计算最终分数
        double timeDecay = calculateTimeDecay(video.getCreatedTime());
        double popularityScore = calculatePopularityScore(video);
        
        return similarityScore * 0.6 +      // 相似度权重
               timeDecay * 0.2 +            // 时间衰减权重
               popularityScore * 0.2;        // 热度权重
    }

    private double calculateTimeDecay(LocalDateTime createdTime) {
        long days = ChronoUnit.DAYS.between(createdTime, LocalDateTime.now());
        return 1.0 / (1.0 + Math.log1p(days));
    }

    private double calculatePopularityScore(Video video) {
        return Math.log1p(video.getViewCount()) + Math.log1p(video.getLikeCount() * 2);
    }

    private double calculateCosineSimilarity(List<VideoTagWeight> tags1, List<VideoTagWeight> tags2) {
        // 实现余弦相似度计算
        return 0.0;
    }
} 